package BatchProcess "Library with an example to demonstrate combination of state chart and fluid model"
  model BatchProcessModel "Model of the batch reactor coupled with a state machine"
    Components.Tank reactants(height = 10, levelStart = 9.5) annotation(
        Placement(visible = true, transformation(origin = {-120, 50}, extent = {{-20, -20}, {20, 20}}))
      );
    Components.Vessel reaction(use_HeatTransfer = true, numPorts = 2, levelStart = 0.001) annotation(
        Placement(visible = true, transformation(origin = {0, 50}, extent = {{-20, -20}, {20, 20}}))
      );
    Components.Tank products(levelStart = 0.001) annotation(
        Placement(visible = true, transformation(origin = {120, 50}, extent = {{-20, -20}, {20, 20}}))
      );
    Components.Heat heatFlow(heatingSignal = heating.active, coolingSignal = cooling.active, heatingLoad.k = 100000, coolingLoad.k = -100000, heatingMaxLoad = 1000000, coolingMaxLoad = -1000000) annotation(
        Placement(visible = true, transformation(origin = {-50, 50}, extent = {{-10, -10}, {10, 10}}, rotation = -810))
      );
    Components.Valve valve1(valveSignal = filling.active) annotation(
        Placement(visible = true, transformation(origin = {-50, -10}, extent = {{-10, -10}, {10, 10}}))
      );
    Components.Valve valve2(valveSignal = draining.active) annotation(
        Placement(visible = true, transformation(origin = {90, -10}, extent = {{-10, -10}, {10, 10}}))
      );
    Components.Pump pump1(pumpSignal = filling.active) annotation(
        Placement(visible = true, transformation(origin = {-90, -10}, extent = {{-10, -10}, {10, 10}}))
      );
    Components.Pump pump2(pumpSignal = draining.active) annotation(
        Placement(visible = true, transformation(origin = {50, -10}, extent = {{-10, -10}, {10, 10}}))
      );
    Components.IdleState idle(condition = reactants.level > 9, displayText = "until level > 9 m") annotation(
        Placement(visible = true, transformation(origin = {-119.995, -70.005}, extent = {{-20.005, -20.005}, {20.005, 20.005}}))
      );
    Components.ProcessState filling(condition = reactants.level < 1, displayText = "until level < 1 m") annotation(
        Placement(visible = true, transformation(origin = {-59.476, -70}, extent = {{-20.524, -20.524}, {20.524, 20.524}}))
      );
    Components.ProcessState heating(condition = reaction.T > 300, displayText = "until T > 300 K") annotation(
        Placement(visible = true, transformation(origin = {1.25, -70}, extent = {{-21.25, -21.25}, {21.25, 21.25}}))
      );
    Components.ProcessState cooling(condition = reaction.T < 295, displayText = "until T < 295 K") annotation(
        Placement(visible = true, transformation(origin = {60, -70}, extent = {{-20, -20}, {20, 20}}))
      );
    Components.ProcessState draining(condition = reaction.level < 1, displayText = "until level < 1 m") annotation(
        Placement(visible = true, transformation(origin = {120, -70}, extent = {{-20, -20}, {20, 20}}))
      );
  equation
    connect(heatFlow.port, reaction.heatPort) annotation(
        Line(visible = true, origin = {-32.403, 50}, points = {{-7.597, 0}, {7.597, -0}}, color = {191, 0, 0})
      );
    connect(valve1.port_b, reaction.ports[1]) annotation(
        Line(visible = true, origin = {-13.085, 1.502}, points = {{-26.915, -11.502}, {13.458, -11.502}, {13.458, 23.003}}, color = {0, 127, 255})
      );
    connect(valve2.port_b, products.ports[1]) annotation(
        Line(visible = true, origin = {113.333, 3.333}, points = {{-13.333, -13.333}, {6.667, -13.333}, {6.667, 26.667}}, color = {0, 127, 255})
      );
    connect(reactants.ports[1], pump1.port_a) annotation(
        Line(visible = true, origin = {-113.333, 3.333}, points = {{-6.667, 26.667}, {-6.667, -13.333}, {13.333, -13.333}}, color = {0, 127, 255})
      );
    connect(pump1.port_b, valve1.port_a) annotation(
        Line(visible = true, origin = {-70.077, -10}, points = {{-10.077, 0}, {10.077, 0}}, color = {0, 127, 255})
      );
    connect(reaction.ports[1], pump2.port_a) annotation(
        Line(visible = true, origin = {13.582, 1.502}, points = {{-13.209, 23.003}, {-13.209, -11.502}, {26.418, -11.502}}, color = {0, 127, 255})
      );
    connect(pump2.port_b, valve2.port_a) annotation(
        Line(visible = true, origin = {69.923, -10}, points = {{-10.077, 0}, {10.077, 0}}, color = {0, 127, 255})
      );
    connect(idle.outPort, filling.inPort) annotation(
        Line(visible = true, origin = {-93.094, -70.002}, points = {{-7.248, -0.002}, {-1.897, -0.002}, {-1.897, 0.002}, {11.042, 0.002}})
      );
    connect(filling.outPort, heating.inPort) annotation(
        Line(visible = true, origin = {-30.718, -70}, points = {{-8.593, 0}, {8.593, 0}})
      );
    connect(heating.outPort, cooling.inPort) annotation(
        Line(visible = true, origin = {30.064, -70}, points = {{-7.936, 0}, {7.936, 0}})
      );
    connect(cooling.outPort, draining.inPort) annotation(
        Line(visible = true, origin = {88.825, -70}, points = {{-9.175, 0}, {9.175, 0}})
      );
    connect(idle.inPort, draining.outPort) annotation(
        Line(visible = true, origin = {3.535, -61.07}, points = {{-145.535, -8.935}, {-163.535, -8.935}, {-163.535, 21.07}, {156.465, 21.07}, {156.465, -8.93}, {136.114, -8.93}})
      );
    annotation(
      experiment(
        StopTime = 5000,
        __Wolfram_Algorithm = "cvodes"
      ),
      Documentation(
        figures = {
          Figure(
            title = "results",
            identifier = "sharp-abel",
            preferred = true,
            plots = {
              Plot(
                curves = {
                  Curve(y = reactants.tankLevel.y),
                  Curve(y = reaction.tankLevel.y),
                  Curve(y = products.tankLevel.y)
                },
                __Wolfram_markerAppearances = {
                  MarkerAppearance(marker = "64385")
                }
              ),
              Plot(
                curves = {
                  Curve(y = reaction.T)
                },
                __Wolfram_markerAppearances = {
                  MarkerAppearance(marker = "64385")
                }
              )
            },
            __Wolfram_markers = {
              Marker(identifier = "64385", position = MarkerPosition(x = 1.116116116116116, xUnit = "s"))
            }
          )
        },
        info = "<html><p>Model consists of a fluid system that will help us understand and test key process dynamics in a controlled environment. The system consists of three tanks, reactant tank stores the reactant, reaction tank is where the process takes place and product tank collects the final product. The reaction tank is equipped with a mechanism for heating or cooling, which is controlled via an external input.</p>
<p class=\"compact-p\">An event-driven behavior is defined for the system, which starts in an idle state. After 5 seconds, the reaction tank begins filling with reactant. Once the reactant level reaches 9 meters, the heating process is initiated. Heating continues until the temperature inside the tank reaches 300 Kelvin, at which point cooling begins. Cooling proceeds until the temperature drops to 295 Kelvin, after which the reaction tank begins to drain.</p>
<p class=\"compact-p\">The state machine governs the behavior of system components by opening and closing valves, starting and stopping pumps, and activating heaters. This hybrid setup of discrete control and physical models effectively replicates real-world industrial process automation.</p>
<p>In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at&nbsp;<a href=\"http://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a></p>
<p>For the full example, open the accompanying notebook&nbsp;<a href=\"modelica://BatchProcess/BatchProcess.nb\">BatchProcess.nb</a>.</p></html>"
      ),
      uses(
        Modelica(
          version = "4.0.0"
        )
      ),
      Diagram(
        coordinateSystem(extent = {{-185, -95}, {185, 95}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5}),
        graphics = {
          Text(visible = true, origin = {53.75, 22.908}, textColor = {160, 160, 164}, extent = {{-28.75, -10}, {28.75, 10}}, textString = DynamicSelect("state", if filling.active then "filling" else if heating.active then "heating" else if cooling.active then "cooling" else if draining.active then "draining" else "idling"), fontSize = 40)
        }
      ),
      Icon(
        coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}),
        graphics = {
          Ellipse(visible = true, lineColor = {75, 138, 73}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-100, -100}, {100, 100}}),
          Polygon(visible = true, lineColor = {0, 0, 255}, fillColor = {75, 138, 73}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-36, 60}, {64, 0}, {-36, -60}, {-36, 60}})
        }
      )
    );
  end BatchProcessModel;

  package Components "Sublibrary of components used in the example"
    model Tank "Model of a hydraulic tank"
      Modelica.Fluid.Vessels.OpenTank tank(height = height, crossArea = crossArea, use_portsData = false, nPorts = numPorts, level_start = levelStart, redeclare replaceable package Medium = Modelica.Media.Examples.TwoPhaseWater) annotation(Placement(visible = true, transformation(origin = {0, 20}, extent = {{-20, -20}, {20, 20}})));
      parameter Modelica.Units.SI.Height height = 10 "Height of tank";
      parameter Modelica.Units.SI.Area crossArea = 1 "Area of tank";
      parameter Modelica.Units.SI.Height levelStart = 10 "Start value of tank level";
      Modelica.Fluid.Interfaces.FluidPorts_b[numPorts] ports annotation(Placement(visible = true, transformation(origin = {0, -70}, extent = {{-10, -10}, {10, 10}}, rotation = -90), iconTransformation(origin = {0, -100}, extent = {{-5.334, -20}, {5.334, 20}}, rotation = -90)));
      parameter Integer numPorts = 1 "Number of ports that can be connected to the tank";
      Modelica.Blocks.Sources.RealExpression tankLevel(y = tank.level) annotation(Placement(visible = true, transformation(origin = {70, 40}, extent = {{-10, -10}, {10, 10}})));
      Modelica.Blocks.Interfaces.RealOutput level annotation(Placement(visible = true, transformation(origin = {110, 40}, extent = {{-10, -10}, {10, 10}}), iconTransformation(origin = {110, 50}, extent = {{-10, -10}, {10, 10}})));
      Modelica.Blocks.Sources.RealExpression tankTemp(y = tank.medium.T) annotation(Placement(visible = true, transformation(origin = {70, 20}, extent = {{-10, -10}, {10, 10}})));
      Modelica.Blocks.Interfaces.RealOutput T annotation(Placement(visible = true, transformation(origin = {110, 20}, extent = {{-10, -10}, {10, 10}}), iconTransformation(origin = {110, -50}, extent = {{-10, -10}, {10, 10}})));
      parameter Real minTemp = 290 "minimum temperature" annotation(Dialog(tab = "Visualization", group = "Parameters"));
      parameter Real maxTemp = 299 "maximum temperature" annotation(Dialog(tab = "Visualization", group = "Parameters"));
      inner Modelica.Fluid.System system annotation(
          Placement(visible = true, transformation(origin = {-90, 50}, extent = {{-10, -10}, {10, 10}}))
        );
    protected
      Real colorMap[3] = BatchProcess.Utilities.temperatureToRGB(minTemp, maxTemp, T);
    equation
      connect(tank.ports, ports) annotation(Line(visible = true, origin = {0, -35}, points = {{0, 35}, {0, -35}}, color = {0, 127, 255}));
      connect(tankLevel.y, level) annotation(Line(visible = true, origin = {95.5, 40}, points = {{-14.5, 0}, {14.5, 0}}, color = {0, 0, 127}));
      connect(tankTemp.y, T) annotation(Line(visible = true, origin = {95.5, 20}, points = {{-14.5, 0}, {14.5, 0}}, color = {0, 0, 127}));
      annotation(
        Documentation(
          info = "<html><p class=\"\" data-start=\"121\" data-end=\"321\">A simple hydraulic tank model used to store fluid. It provides level and temperature outputs and supports visual temperature mapping using RGB colors. Ideal for basic storage and monitoring scenarios.</p></html>"
        ),
        Diagram(coordinateSystem(extent = {{-150, -90}, {150, 90}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {85, 85, 85}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-100, -100}, {100, 100}}), Text(visible = true, origin = {0, 43.969}, textColor = {160, 160, 164}, extent = {{-150, 110}, {150, 150}}, textString = "%name"), Rectangle(visible = true, origin = {0, 2.245}, lineColor = DynamicSelect({85, 85, 85}, colorMap), fillColor = DynamicSelect({77, 166, 255}, colorMap), fillPattern = FillPattern.Solid, extent = DynamicSelect({{-100, -102.245}, {100, 10.225}}, {{-97.653, -100}, {97.653, -100 + 200 * level / height}})), Line(visible = true, points = {{-100, 100}, {-100, -100}, {100, -100}, {100, 100}}, color = {85, 85, 85}, thickness = 3), Text(visible = true, origin = {145.057, 25.837}, extent = {{-26.842, -15}, {26.842, 15}}, textString = DynamicSelect("%{levelStart}", integer(level)), fontSize = DynamicSelect(0, 75), horizontalAlignment = TextAlignment.Right), Text(visible = true, origin = {144.468, -72.045}, extent = {{-26.842, -15}, {26.842, 15}}, textString = DynamicSelect("", integer(T)), fontSize = DynamicSelect(0, 75), horizontalAlignment = TextAlignment.Right), Text(visible = true, origin = {206.842, -72.575}, extent = {{-26.842, -15}, {26.842, 15}}, textString = DynamicSelect("", "K"), fontSize = DynamicSelect(0, 75), horizontalAlignment = DynamicSelect(TextAlignment.Left, TextAlignment.Left)), Text(visible = true, origin = {208.379, 27.099}, extent = {{-26.842, -15}, {26.842, 15}}, textString = DynamicSelect("", "m"), fontSize = DynamicSelect(0, 75), horizontalAlignment = DynamicSelect(TextAlignment.Left, TextAlignment.Left)), Polygon(visible = true, origin = {0, 113.333}, lineColor = {85, 85, 85}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 3, points = {{-100, -13.333}, {100, -13.333}, {100, -9.524}, {0, 26.667}, {-100, -10.353}})}));
    end Tank;

    model Vessel "Model of a vessel with heating or cooling jacket"
      Modelica.Fluid.Vessels.OpenTank tank(height = height, crossArea = crossArea, use_portsData = false, nPorts = numPorts, level_start = levelStart, redeclare replaceable package Medium = Modelica.Media.Examples.TwoPhaseWater, use_HeatTransfer = use_HeatTransfer) annotation(Placement(visible = true, transformation(origin = {0, 20}, extent = {{-20, -20}, {20, 20}})));
      parameter Modelica.Units.SI.Height height = 10 "Height of tank";
      parameter Modelica.Units.SI.Area crossArea = 1 "Area of tank";
      parameter Modelica.Units.SI.Height levelStart = 10 "Start value of tank level";
      parameter Boolean use_HeatTransfer = false "= true to use the HeatTransfer model";
      Modelica.Fluid.Interfaces.FluidPorts_b[numPorts] ports annotation(Placement(visible = true, transformation(origin = {0, -70}, extent = {{-10, -10}, {10, 10}}, rotation = -90), iconTransformation(origin = {1.883, -127.477}, extent = {{-5.334, -20.204}, {5.334, 20.204}}, rotation = -90)));
      parameter Integer numPorts = 1 "Number of ports that can be connected to the tank";
      Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort if use_HeatTransfer annotation(Placement(visible = true, transformation(origin = {-60, 20}, extent = {{-10, -10}, {10, 10}}), iconTransformation(origin = {-124.03, 0}, extent = {{-10, -10}, {10, 10}})));
      Modelica.Blocks.Sources.RealExpression tankLevel(y = tank.level) annotation(Placement(visible = true, transformation(origin = {70, 40}, extent = {{-10, -10}, {10, 10}})));
      Modelica.Blocks.Interfaces.RealOutput level annotation(Placement(visible = true, transformation(origin = {110, 40}, extent = {{-10, -10}, {10, 10}}), iconTransformation(origin = {132.391, 44.887}, extent = {{-10, -10}, {10, 10}})));
      Modelica.Blocks.Sources.RealExpression tankTemp(y = tank.medium.T) annotation(Placement(visible = true, transformation(origin = {70, 20}, extent = {{-10, -10}, {10, 10}})));
      Modelica.Blocks.Interfaces.RealOutput T annotation(Placement(visible = true, transformation(origin = {110, 20}, extent = {{-10, -10}, {10, 10}}), iconTransformation(origin = {132.496, -42.842}, extent = {{-10, -10}, {10, 10}})));
      parameter Real minTemp = 290 "minimum temperature" annotation(Dialog(tab = "Visualization", group = "Parameters"));
      parameter Real maxTemp = 299 "maximum temperature" annotation(Dialog(tab = "Visualization", group = "Parameters"));
      inner Modelica.Fluid.System system annotation(
          Placement(visible = true, transformation(origin = {-90, 50}, extent = {{-10, -10}, {10, 10}}))
        );
    protected
      Real colorMap[3] = BatchProcess.Utilities.temperatureToRGB(minTemp, maxTemp, T);
    equation
      connect(tank.ports, ports) annotation(Line(visible = true, origin = {0, -35}, points = {{0, 35}, {0, -35}}, color = {0, 127, 255}));
      connect(tank.heatPort, heatPort) annotation(Line(visible = true, origin = {-40, 20}, points = {{20, 0}, {-20, 0}}, color = {191, 0, 0}));
      connect(tankLevel.y, level) annotation(Line(visible = true, origin = {95.5, 40}, points = {{-14.5, 0}, {14.5, 0}}, color = {0, 0, 127}));
      connect(tankTemp.y, T) annotation(Line(visible = true, origin = {95.5, 20}, points = {{-14.5, 0}, {14.5, 0}}, color = {0, 0, 127}));
      annotation(
        Documentation(
          info = "<html><p class=\"\" data-start=\"346\" data-end=\"528\">An enhanced hydraulic tank model with optional heat transfer interaction. It supports external heating or cooling via a heat port, making it suitable for thermal process simulations.</p></html>"
        ),
        Diagram(coordinateSystem(extent = {{-150, -90}, {150, 90}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Ellipse(visible = true, origin = {0.872, -105.052}, lineColor = {85, 85, 85}, fillColor = DynamicSelect({77, 166, 255}, colorMap), fillPattern = FillPattern.Solid, lineThickness = 3, extent = {{-100.872, -20}, {100.872, 20}}), Ellipse(visible = true, origin = {0.872, 102.662}, lineColor = {85, 85, 85}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 3, extent = {{-100.872, -25.218}, {100.872, 25.218}}), Rectangle(visible = true, origin = {0.872, -2.042}, lineColor = {85, 85, 85}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, lineThickness = 3, extent = {{-100.872, -102.042}, {100.872, 102.042}}), Text(visible = true, origin = {0, 36.812}, textColor = {160, 160, 164}, extent = {{-150, 110}, {150, 150}}, textString = "%name"), Rectangle(visible = true, origin = {0.943, -0.659}, lineColor = DynamicSelect({85, 85, 85}, colorMap), fillColor = DynamicSelect({77, 166, 255}, colorMap), pattern = LinePattern.None, fillPattern = FillPattern.Solid, extent = DynamicSelect({{-99.654, -106.59}, {99.654, 10.659}}, {{-99.654, -106.59}, {99.654, -100 + 200 * level / height}})), Text(visible = true, origin = {174.804, 25}, extent = {{-26.842, -15}, {26.842, 15}}, textString = DynamicSelect("%{levelStart}", integer(level)), fontSize = DynamicSelect(0, 75), horizontalAlignment = TextAlignment.Right), Text(visible = true, origin = {178.651, -72.882}, extent = {{-26.842, -15}, {26.842, 15}}, textString = DynamicSelect("", integer(T)), fontSize = DynamicSelect(0, 75), horizontalAlignment = TextAlignment.Right), Text(visible = true, origin = {235.668, -72.575}, extent = {{-26.842, -15}, {26.842, 15}}, textString = DynamicSelect("", "K"), fontSize = DynamicSelect(0, 75), horizontalAlignment = DynamicSelect(TextAlignment.Left, TextAlignment.Left)), Text(visible = true, origin = {232.51, 25}, extent = {{-26.842, -15}, {26.842, 15}}, textString = DynamicSelect("", "m"), fontSize = DynamicSelect(0, 75), horizontalAlignment = DynamicSelect(TextAlignment.Left, TextAlignment.Left)), Rectangle(visible = true, origin = {-111, 0}, lineColor = {85, 85, 85}, fillColor = DynamicSelect({255, 255, 255}, if heatPort.Q_flow > 10 then {247, 135, 115} else if heatPort.Q_flow < -10 then {48, 175, 244} else {255, 255, 255}), fillPattern = FillPattern.Solid, lineThickness = 3, extent = {{-10, -60}, {10, 60}}), Rectangle(visible = true, origin = {112, 2}, lineColor = {85, 85, 85}, fillColor = DynamicSelect({255, 255, 255}, if heatPort.Q_flow > 10 then {247, 135, 115} else if heatPort.Q_flow < -10 then {48, 175, 244} else {255, 255, 255}), fillPattern = FillPattern.Solid, lineThickness = 3, extent = {{-9.098, -60}, {9.098, 60}})}));
    end Vessel;

    model Valve "Model of a discrete valve"
      Modelica.Fluid.Valves.ValveDiscrete valve(m_flow_nominal = 1, dp_nominal = 10000, redeclare replaceable package Medium = Modelica.Media.Examples.TwoPhaseWater, opening_min = 0.001) annotation(
          Placement(visible = true, transformation(origin = {0, -0}, extent = {{-10, -10}, {10, 10}}))
        );
      Modelica.Fluid.Interfaces.FluidPort_b port_b annotation(
          Placement(visible = true, transformation(origin = {60, 0}, extent = {{-10, -10}, {10, 10}}), iconTransformation(origin = {100, -0}, extent = {{-10, -10}, {10, 10}}))
        );
      Modelica.Fluid.Interfaces.FluidPort_a port_a annotation(
          Placement(visible = true, transformation(origin = {-61.519, 0}, extent = {{-10, -10}, {10, 10}}), iconTransformation(origin = {-100, -0}, extent = {{-10, -10}, {10, 10}}))
        );
      Modelica.Blocks.Sources.BooleanExpression valveON(y = valveSignal) annotation(
          Placement(visible = true, transformation(origin = {-50, 40}, extent = {{-10, -10}, {10, 10}}))
        );
      input Boolean valveSignal (start = false) "= true, if transition may fire (time varying expression)" annotation(Dialog(group = "Fire condition"));
      inner Modelica.Fluid.System system annotation(
          Placement(visible = true, transformation(origin = {-80, 50}, extent = {{-10, -10}, {10, 10}}))
        );
    equation
      connect(valve.port_b, port_b) annotation(
          Line(visible = true, origin = {35, 0}, points = {{-25, 0}, {25, 0}}, color = {0, 127, 255})
        );
      connect(valve.port_a, port_a) annotation(
          Line(visible = true, origin = {-35.759, 0}, points = {{25.759, 0}, {-25.76, 0}}, color = {0, 127, 255})
        );
      connect(valveON.y, valve.open) annotation(
          Line(visible = true, origin = {-13, 29.333}, points = {{-26, 10.667}, {13, 10.667}, {13, -21.333}}, color = {255, 0, 255})
        );
      annotation(
        Documentation(
          info = "<html><p class=\"\" data-start=\"552\" data-end=\"732\">A discrete two-way valve that opens or closes based on a boolean signal. It controls fluid flow through the connected ports and is ideal for on/off flow control in process systems.</p></html>"
        ),
        Diagram(
          coordinateSystem(extent = {{-100, -40}, {100, 70}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})
        ),
        Icon(
          coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}),
          graphics = {
            Text(visible = true, textColor = {160, 160, 164}, extent = {{-150, 110}, {150, 150}}, textString = "%name"),
            Polygon(visible = true, lineColor = {85, 85, 85}, fillColor = DynamicSelect({211, 211, 211}, if valveSignal then {50, 205, 50} else {211, 211, 211}), fillPattern = FillPattern.Solid, lineThickness = 5, points = {{-100, 50}, {100, -50}, {100, 50}, {0, 0}, {-100, -50}, {-100, 50}})
          }
        )
      );
    end Valve;

    model Pump "Model of a simple pump"
      Modelica.Fluid.Interfaces.FluidPort_b port_b annotation(
          Placement(visible = true, transformation(origin = {60, 0}, extent = {{-10, -10}, {10, 10}}), iconTransformation(origin = {98.466, 0}, extent = {{-10, -10}, {10, 10}}))
        );
      Modelica.Fluid.Interfaces.FluidPort_a port_a annotation(
          Placement(visible = true, transformation(origin = {-61.519, 0}, extent = {{-10, -10}, {10, 10}}), iconTransformation(origin = {-100, 0}, extent = {{-10, -10}, {10, 10}}))
        );
      Modelica.Fluid.Machines.Pump pump(redeclare replaceable package Medium = Modelica.Media.Examples.TwoPhaseWater, redeclare replaceable function flowCharacteristic = Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.linearFlow(V_flow_nominal = {0, 0.5}, head_nominal = {1, 0.1}), N_nominal = 1000) annotation(
          Placement(visible = true, transformation(extent = {{-10, -10}, {10, 10}}))
        );
      Modelica.Mechanics.Rotational.Sources.Speed speed1 annotation(
          Placement(visible = true, transformation(origin = {-30, 40}, extent = {{-10, -10}, {10, 10}}))
        );
      Modelica.Blocks.Logical.Switch pumpSpeed annotation(
          Placement(visible = true, transformation(origin = {-70, 40}, extent = {{-10, -10}, {10, 10}}))
        );
      Modelica.Blocks.Sources.Constant noLoad(k = 0.001) annotation(
          Placement(visible = true, transformation(origin = {-120, 10}, extent = {{-10, -10}, {10, 10}}))
        );
      Modelica.Blocks.Sources.Constant nominalSpeed(k = 350) annotation(
          Placement(visible = true, transformation(origin = {-120, 60}, extent = {{-10, -10}, {10, 10}}))
        );
      Modelica.Blocks.Sources.BooleanExpression pumpON(y = pumpSignal) annotation(
          Placement(visible = true, transformation(origin = {-150, 40}, extent = {{-10, -10}, {10, 10}}))
        );
        input Boolean pumpSignal (start = false) "= true, if transition may fire (time varying expression)" annotation(Dialog(group = "Fire condition"));
      inner Modelica.Fluid.System system annotation(
          Placement(visible = true, transformation(origin = {-150, -30}, extent = {{-10, -10}, {10, 10}}))
        );
    equation
      connect(port_a, pump.port_a) annotation(
          Line(visible = true, origin = {-35.76, 0}, points = {{-25.76, 0}, {25.76, 0}}, color = {0, 127, 255}));
      connect(pump.port_b, port_b) annotation(
          Line(visible = true, origin = {35, 0}, points = {{-25, 0}, {25, 0}}, color = {0, 127, 255})
        );
      connect(speed1.flange, pump.shaft) annotation(
          Line(visible = true, origin = {-6.667, 30}, points = {{-13.333, 10}, {6.667, 10}, {6.667, -20}})
        );
      connect(noLoad.y, pumpSpeed.u3) annotation(
          Line(visible = true, origin = {-97.75, 21}, points = {{-11.25, -11}, {-2.25, -11}, {-2.25, 11}, {15.75, 11}}, color = {0, 0, 127})
        );
      connect(nominalSpeed.y, pumpSpeed.u1) annotation(
          Line(visible = true, origin = {-97.75, 54}, points = {{-11.25, 6}, {-2.25, 6}, {-2.25, -6}, {15.75, -6}}, color = {0, 0, 127})
        );
      connect(pumpON.y, pumpSpeed.u2) annotation(
          Line(visible = true, origin = {-110.5, 40}, points = {{-28.5, 0}, {28.5, 0}}, color = {255, 0, 255})
        );
      connect(pumpSpeed.y, speed1.w_ref) annotation(
          Line(visible = true, origin = {-50.5, 40}, points = {{-8.5, 0}, {8.5, 0}}, color = {0, 0, 127})
        );
      annotation(
        Documentation(
          info = "<html><p class=\"\" data-start=\"755\" data-end=\"947\">A dynamic pump model that moves fluid based on a speed-controlled shaft. It switches between no-load and nominal speed using a control signal, simulating a realistic fluid transport mechanism.</p></html>"
        ),
        __Wolfram(
          itemFlippingEnabled = true
        ),
        Diagram(
          coordinateSystem(extent = {{-167.824, -55}, {100, 90}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})
        ),
        Icon(
          coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}),
          graphics = {
            Text(visible = true, textColor = {160, 160, 164}, extent = {{-150, 110}, {150, 150}}, textString = "%name"),
            Ellipse(visible = true, origin = {-1.123, -1.534}, lineColor = {85, 85, 85}, fillColor = DynamicSelect({176, 176, 176}, if pumpSignal then {34, 139, 34} else {176, 176, 176}), fillPattern = FillPattern.Solid, lineThickness = 5, extent = {{-101.123, -100}, {101.123, 100}}),
            Line(visible = true, origin = {-12.752, -0.696}, points = {{-57.248, 70.696}, {114.495, 0.696}, {-57.248, -71.393}}, color = {85, 85, 85}, thickness = 5)
          }
        )
      );
    end Pump;

    model IdleState "Model of the idle state"
      Modelica.StateGraph.InitialStep idle(nIn = 1, nOut = 1) annotation(
          Placement(visible = true, transformation(origin = {8, -10}, extent = {{-48, 0}, {-28, 20}}))
        );
      Modelica.StateGraph.Transition transition1(enableTimer = true, waitTime = waitTime, condition = condition) annotation(
          Placement(visible = true, transformation(origin = {10, -10}, extent = {{-20, 0}, {0, 20}}))
        );
      Modelica.StateGraph.Interfaces.Step_in inPort annotation(
          Placement(visible = true, transformation(origin = {-70, -0}, extent = {{-10, -10}, {10, 10}}), iconTransformation(origin = {-110, 0}, extent = {{-10, -10}, {10, 10}}))
        );
      Modelica.StateGraph.Interfaces.Transition_out outPort annotation(
          Placement(visible = true, transformation(origin = {35, 0}, extent = {{-10, -10}, {10, 10}}), iconTransformation(origin = {110, 0}, extent = {{-10, -10}, {10, 10}}))
        );
      input Boolean condition  "= true, if transition may fire (time varying expression)" annotation(Dialog(group = "Fire condition")
        );
      parameter Modelica.Units.SI.Time waitTime(min = 0) = 5 "Wait time before transition fires (transition1.waitTime)";
      parameter String displayText "Text displayed in diagrams";
      inner Modelica.StateGraph.StateGraphRoot stateGraphRoot annotation(
          Placement(visible = true, transformation(origin = {-60, -25}, extent = {{-10, -10}, {10, 10}}))
        );
    equation
      connect(idle.inPort[1], inPort) annotation(
          Line(visible = true, origin = {-55.5, -0}, points = {{14.5, 0}, {-14.5, -0}})
        );
      connect(transition1.outPort, outPort) annotation(
          Line(visible = true, origin = {18.25, 0}, points = {{-16.75, -0}, {16.75, 0}})
        );
      connect(idle.outPort[1], transition1.inPort) annotation(
          Line(visible = true, origin = {-11.75, 0}, points = {{-7.75, 0}, {7.75, 0}})
        );
      annotation(
        Documentation(
          info = "<html><p class=\"\" data-start=\"975\" data-end=\"1165\">Represents an initial idle state in a process workflow using the StateGraph library. It waits for a specified time and a condition before transitioning, useful for startup or standby stages.</p></html>"
        ),
        Diagram(
          coordinateSystem(extent = {{-80, -40}, {55, 40}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})
        ),
        Icon(
          coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}),
          graphics = {
            Rectangle(visible = true, lineColor = {0, 114, 195}, fillColor = DynamicSelect({255, 255, 255}, if condition then {211, 211, 211} else {255, 255, 255}), fillPattern = FillPattern.Solid, extent = {{-100, -50}, {100, 50}}, radius = 25),
            Text(visible = true, origin = {0, -50}, textColor = {160, 160, 164}, extent = {{-150, 110}, {150, 150}}, textString = "%name"),
            Text(visible = true, origin = {2.486, 1.614}, textColor = {83, 83, 85}, extent = {{-87.514, -38.386}, {87.514, 38.386}}, textString = "%displayText", fontSize = 72)
          }
        )
      );
    end IdleState;

    model ProcessState "Model of a process with conditions"
      Modelica.StateGraph.StepWithSignal filling(nIn = 1, nOut = 1) annotation(
          Placement(visible = true, transformation(origin = {-50, -10}, extent = {{10, 0}, {30, 20}}))
        );
      Modelica.StateGraph.Transition transition2(enableTimer = true, waitTime = waitTime, condition = condition) annotation(
          Placement(visible = true, transformation(origin = {10, -10}, extent = {{-20, 0}, {0, 20}}))
        );
      Modelica.StateGraph.Interfaces.Step_in inPort annotation(
          Placement(visible = true, transformation(origin = {-70, 0}, extent = {{-10, -10}, {10, 10}}), iconTransformation(origin = {-110, 0}, extent = {{-10, -10}, {10, 10}}))
        );
      Modelica.StateGraph.Interfaces.Transition_out outPort annotation(
          Placement(visible = true, transformation(origin = {35, -0}, extent = {{-10, -10}, {10, 10}}), iconTransformation(origin = {110, 0}, extent = {{-10, -10}, {10, 10}}))
        );
      input Boolean condition  "= true, if transition may fire (time varying expression)" annotation(Dialog(group = "Fire condition")
        );
      parameter Modelica.Units.SI.Time waitTime(min = 0) = 5 "Wait time before transition fires (transition2.waitTime)";
      Modelica.Blocks.Interfaces.BooleanOutput active annotation(
          Placement(visible = true, transformation(origin = {-30, -30}, extent = {{-10, -10}, {10, 10}}, rotation = -90), iconTransformation(origin = {0, -60}, extent = {{-10, -10}, {10, 10}}, rotation = -90))
        );
      parameter String displayText "Text displayed in diagrams";
      inner Modelica.StateGraph.StateGraphRoot stateGraphRoot annotation(
          Placement(visible = true, transformation(origin = {-60, -30}, extent = {{-10, -10}, {10, 10}}))
        );
    equation
      connect(transition2.inPort, filling.outPort[1]) annotation(
          Line(visible = true, origin = {-11.75, 0}, points = {{7.75, 0}, {-7.75, 0}})
        );
      connect(inPort, filling.inPort[1]) annotation(
          Line(visible = true, origin = {-55.5, 0}, points = {{-14.5, 0}, {14.5, -0}})
        );
      connect(transition2.outPort, outPort) annotation(
          Line(visible = true, origin = {18.25, -0}, points = {{-16.75, 0}, {16.75, -0}})
        );
      connect(filling.active, active) annotation(
          Line(visible = true, origin = {-30, -20.5}, points = {{0, 9.5}, {-0, -9.5}}, color = {255, 0, 255})
        );
      annotation(
        Documentation(
          info = "<html><p class=\"\" data-start=\"1196\" data-end=\"1389\">Defines an active state within a process sequence, signaling when it's engaged. It includes timing and condition-based transitions, ideal for modeling filling, heating, or other process phases.</p></html>"
        ),
        Diagram(
          coordinateSystem(extent = {{-80, -50}, {60, 40}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})
        ),
        Icon(
          coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}),
          graphics = {
            Rectangle(visible = true, lineColor = {0, 114, 195}, fillColor = DynamicSelect({255, 255, 255}, if active then {211, 211, 211} else {255, 255, 255}), fillPattern = FillPattern.Solid, extent = {{-100, -50}, {100, 50}}, radius = 25),
            Text(visible = true, origin = {0, -50}, textColor = {160, 160, 164}, extent = {{-150, 110}, {150, 150}}, textString = "%name"),
            Text(visible = true, origin = {0, -2.071}, textColor = {83, 83, 85}, extent = {{-86.472, -37.929}, {86.472, 37.929}}, textString = "%displayText", fontSize = 72)
          }
        )
      );
    end ProcessState;

    model Heat "Heating or cooling source"
      input Boolean heatingSignal(start = true) "= true, if transition may fire (time varying expression)" annotation(Dialog(group = "Fire condition"));
      input Boolean coolingSignal(start = false) "= true, if transition may fire (time varying expression)" annotation(Dialog(group = "Fire condition"));
      Modelica.Thermal.HeatTransfer.Sources.PrescribedHeatFlow heat annotation(Placement(visible = true, transformation(origin = {30, 0}, extent = {{-10, -10}, {10, 10}})));
      Modelica.Blocks.Logical.Switch heatLoad annotation(Placement(visible = true, transformation(origin = {-70, 30}, extent = {{-10, -10}, {10, 10}})));
      Modelica.Blocks.Math.Add neatHeat annotation(Placement(visible = true, transformation(origin = {-10, 0}, extent = {{-10, -10}, {10, 10}})));
      Modelica.Blocks.Logical.Switch coolLoad annotation(Placement(visible = true, transformation(origin = {-70, -30}, extent = {{-10, 10}, {10, -10}})));
      Modelica.Blocks.Sources.Constant noLoad annotation(Placement(visible = true, transformation(origin = {-120, 0}, extent = {{-10, -10}, {10, 10}})));
      Modelica.Blocks.Sources.Constant heatingLoad(k = heatingMaxLoad) annotation(Placement(visible = true, transformation(origin = {-120, 50}, extent = {{-10, -10}, {10, 10}})));
      Modelica.Blocks.Sources.Constant coolingLoad(k = coolingMaxLoad) annotation(Placement(visible = true, transformation(origin = {-120, -50}, extent = {{-10, -10}, {10, 10}})));
      Modelica.Blocks.Sources.BooleanExpression heatingON(y = heatingSignal) annotation(Placement(visible = true, transformation(origin = {-150, 30}, extent = {{-10, -10}, {10, 10}})));
      Modelica.Blocks.Sources.BooleanExpression coolingON(y = coolingSignal) annotation(Placement(visible = true, transformation(origin = {-150, -30}, extent = {{-10, -10}, {10, 10}})));
      Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b port annotation(Placement(visible = true, transformation(origin = {80, 0}, extent = {{-10, -10}, {10, 10}}), iconTransformation(origin = {-50, 100}, extent = {{-10, -10}, {10, 10}})));
      parameter Real heatingMaxLoad(start = 1) = 10000 "Constant output value (heatingLoad.k)";
      parameter Real coolingMaxLoad(start = 1) = -10000 "Constant output value (coolingLoad.k)";
      Real y = time;
    equation
      connect(noLoad.y, heatLoad.u3) annotation(Line(visible = true, origin = {-97.75, 11}, points = {{-11.25, -11}, {-2.25, -11}, {-2.25, 11}, {15.75, 11}}, color = {0, 0, 127}));
      connect(coolLoad.u3, noLoad.y) annotation(Line(visible = true, origin = {-97.75, -11}, points = {{15.75, -11}, {-2.25, -11}, {-2.25, 11}, {-11.25, 11}}, color = {0, 0, 127}));
      connect(coolLoad.y, neatHeat.u2) annotation(Line(visible = true, origin = {-40.25, -18}, points = {{-18.75, -12}, {0.25, -12}, {0.25, 12}, {18.25, 12}}, color = {0, 0, 127}));
      connect(neatHeat.y, heat.Q_flow) annotation(Line(visible = true, origin = {10.5, 0}, points = {{-9.5, 0}, {9.5, 0}}, color = {0, 0, 127}));
      connect(neatHeat.u1, heatLoad.y) annotation(Line(visible = true, origin = {-40.25, 18}, points = {{18.25, -12}, {0.25, -12}, {0.25, 12}, {-18.75, 12}}, color = {0, 0, 127}));
      connect(heatingLoad.y, heatLoad.u1) annotation(Line(visible = true, origin = {-97.75, 44}, points = {{-11.25, 6}, {-2.25, 6}, {-2.25, -6}, {15.75, -6}}, color = {0, 0, 127}));
      connect(coolingLoad.y, coolLoad.u1) annotation(Line(visible = true, origin = {-97.75, -44}, points = {{-11.25, -6}, {-2.25, -6}, {-2.25, 6}, {15.75, 6}}, color = {0, 0, 127}));
      connect(heatingON.y, heatLoad.u2) annotation(Line(visible = true, origin = {-110.5, 30}, points = {{-28.5, 0}, {28.5, 0}}, color = {255, 0, 255}));
      connect(coolingON.y, coolLoad.u2) annotation(Line(visible = true, origin = {-110.5, -30}, points = {{-28.5, 0}, {28.5, 0}}, color = {255, 0, 255}));
      connect(heat.port, port) annotation(Line(visible = true, origin = {60, 0}, points = {{-20, 0}, {20, 0}}, color = {191, 0, 0}));
      annotation(
        Documentation(
          info = "<html><p class=\"\" data-start=\"1412\" data-end=\"1603\">A heat source component that applies prescribed heating or cooling based on two input signals. It dynamically adjusts the heat flow, useful for simulating thermal control in tanks or vessels.</p></html>"
        ),
        Diagram(coordinateSystem(extent = {{-175, -90}, {100, 90}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})),Icon(
          coordinateSystem(extent = {{-170, -148.473}, {70, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5}),
          graphics = {
            Text(visible = true, origin = {-50, -325.722}, textColor = {160, 160, 164}, extent = {{-150, 110}, {150, 150}}, textString = "%name"),
            Line(visible = true, origin = {-70, -164.278}, rotation = -450, points = {{-20, 140}, {-20, -100}}, color = {230, 46, 46}, thickness = 25),
            Line(visible = true, origin = {-75, -10}, points = DynamicSelect(
              {{-52.794, 60}, {-57.975, 53.6}, {-60, 47.2}, {-58.066, 40.8}, {-52.94, 34.4}, {-46.65, 28}, {-41.686, 21.6}, {-40.013, 15.2}, {-42.293, 8.8}, {-47.623, 2.4}, {-53.894, -4}, {-58.624, -10.4}, {-59.94, -16.8}, {-57.322, -23.2}, {-51.806, -29.6}, {-45.575, -36}, {-41.095, -42.4}, {-40.14, -48.8}, {-43.088, -55.2}, {-48.771, -61.6}, {-54.941, -68}, {-59.155, -74.4}, {-59.746, -80.8}, {-56.479, -87.2}, {-50.647, -93.6}, {-44.56, -100}},
              {{((-50.) + (10. * sin((6. + ((-0.005) * y))))), 60.}, {((-50.) + (10. * sin((5.36 + ((-0.005) * y))))), 53.6}, {((-50.) + (10. * sin((4.72 + ((-0.005) * y))))), 47.2}, {((-50.) + (10. * sin((4.08 + ((-0.005) * y))))), 40.8}, {((-50.) + (10. * sin((3.44 + ((-0.005) * y))))), 34.4}, {((-50.) + (10. * sin((2.8 + ((-0.005) * y))))), 28.}, {((-50.) + (10. * sin((2.16 + ((-0.005) * y))))), 21.6}, {((-50.) + (10. * sin((1.52 + ((-0.005) * y))))), 15.2}, {((-50.) + (10. * sin((0.88 + ((-0.005) * y))))), 8.8}, {((-50.) + (10. * sin((0.24 + ((-0.005) * y))))), 2.4}, {((-50.) + ((-10.) * sin((0.4 + (0.005 * y))))), (-4.)}, {((-50.) + ((-10.) * sin((1.04 + (0.005 * y))))), (-10.4)}, {((-50.) + ((-10.) * sin((1.68 + (0.005 * y))))), (-16.8)}, {((-50.) + ((-10.) * sin((2.32 + (0.005 * y))))), (-23.2)}, {((-50.) + ((-10.) * sin((2.96 + (0.005 * y))))), (-29.6)}, {((-50.) + ((-10.) * sin((3.6 + (0.005 * y))))), (-36.)}, {((-50.) + ((-10.) * sin((4.24 + (0.005 * y))))), (-42.4)}, {((-50.) + ((-10.) * sin((4.88 + (0.005 * y))))), (-48.8)}, {((-50.) + ((-10.) * sin((5.52 + (0.005 * y))))), (-55.2)}, {((-50.) + ((-10.) * sin((6.16 + (0.005 * y))))), (-61.6)}, {((-50.) + ((-10.) * sin((6.8 + (0.005 * y))))), (-68.)}, {((-50.) + ((-10.) * sin((7.44 + (0.005 * y))))), (-74.4)}, {((-50.) + ((-10.) * sin((8.08 + (0.005 * y))))), (-80.8)}, {((-50.) + ((-10.) * sin((8.72 + (0.005 * y))))), (-87.2)}, {((-50.) + ((-10.) * sin((9.36 + (0.005 * y))))), (-93.6)}, {((-50.) + ((-10.) * sin((10. + (0.005 * y))))), (-100.)}})
              , color = DynamicSelect({230, 46, 46}, if heatingSignal then {247, 135, 115} else if coolingSignal then {48, 175, 244} else {255, 255, 255}), thickness = 10, smooth = Smooth.Bezier),
              Line(visible = true, origin = {0, -10}, points = DynamicSelect(
              {{-52.794, 60}, {-57.975, 53.6}, {-60, 47.2}, {-58.066, 40.8}, {-52.94, 34.4}, {-46.65, 28}, {-41.686, 21.6}, {-40.013, 15.2}, {-42.293, 8.8}, {-47.623, 2.4}, {-53.894, -4}, {-58.624, -10.4}, {-59.94, -16.8}, {-57.322, -23.2}, {-51.806, -29.6}, {-45.575, -36}, {-41.095, -42.4}, {-40.14, -48.8}, {-43.088, -55.2}, {-48.771, -61.6}, {-54.941, -68}, {-59.155, -74.4}, {-59.746, -80.8}, {-56.479, -87.2}, {-50.647, -93.6}, {-44.56, -100}},
              {{((-50.) + (10. * sin((6. + ((-0.005) * y))))), 60.}, {((-50.) + (10. * sin((5.36 + ((-0.005) * y))))), 53.6}, {((-50.) + (10. * sin((4.72 + ((-0.005) * y))))), 47.2}, {((-50.) + (10. * sin((4.08 + ((-0.005) * y))))), 40.8}, {((-50.) + (10. * sin((3.44 + ((-0.005) * y))))), 34.4}, {((-50.) + (10. * sin((2.8 + ((-0.005) * y))))), 28.}, {((-50.) + (10. * sin((2.16 + ((-0.005) * y))))), 21.6}, {((-50.) + (10. * sin((1.52 + ((-0.005) * y))))), 15.2}, {((-50.) + (10. * sin((0.88 + ((-0.005) * y))))), 8.8}, {((-50.) + (10. * sin((0.24 + ((-0.005) * y))))), 2.4}, {((-50.) + ((-10.) * sin((0.4 + (0.005 * y))))), (-4.)}, {((-50.) + ((-10.) * sin((1.04 + (0.005 * y))))), (-10.4)}, {((-50.) + ((-10.) * sin((1.68 + (0.005 * y))))), (-16.8)}, {((-50.) + ((-10.) * sin((2.32 + (0.005 * y))))), (-23.2)}, {((-50.) + ((-10.) * sin((2.96 + (0.005 * y))))), (-29.6)}, {((-50.) + ((-10.) * sin((3.6 + (0.005 * y))))), (-36.)}, {((-50.) + ((-10.) * sin((4.24 + (0.005 * y))))), (-42.4)}, {((-50.) + ((-10.) * sin((4.88 + (0.005 * y))))), (-48.8)}, {((-50.) + ((-10.) * sin((5.52 + (0.005 * y))))), (-55.2)}, {((-50.) + ((-10.) * sin((6.16 + (0.005 * y))))), (-61.6)}, {((-50.) + ((-10.) * sin((6.8 + (0.005 * y))))), (-68.)}, {((-50.) + ((-10.) * sin((7.44 + (0.005 * y))))), (-74.4)}, {((-50.) + ((-10.) * sin((8.08 + (0.005 * y))))), (-80.8)}, {((-50.) + ((-10.) * sin((8.72 + (0.005 * y))))), (-87.2)}, {((-50.) + ((-10.) * sin((9.36 + (0.005 * y))))), (-93.6)}, {((-50.) + ((-10.) * sin((10. + (0.005 * y))))), (-100.)}})
              , color = DynamicSelect({230, 46, 46}, if heatingSignal then {247, 135, 115} else if coolingSignal then {48, 175, 244} else {255, 255, 255}), thickness = 10, smooth = Smooth.Bezier),
              Line(visible = true, origin = {75, -20}, points = DynamicSelect(
              {{-52.794, 60}, {-57.975, 53.6}, {-60, 47.2}, {-58.066, 40.8}, {-52.94, 34.4}, {-46.65, 28}, {-41.686, 21.6}, {-40.013, 15.2}, {-42.293, 8.8}, {-47.623, 2.4}, {-53.894, -4}, {-58.624, -10.4}, {-59.94, -16.8}, {-57.322, -23.2}, {-51.806, -29.6}, {-45.575, -36}, {-41.095, -42.4}, {-40.14, -48.8}, {-43.088, -55.2}, {-48.771, -61.6}, {-54.941, -68}, {-59.155, -74.4}, {-59.746, -80.8}, {-56.479, -87.2}, {-50.647, -93.6}, {-44.56, -100}},
              {{((-50.) + (10. * sin((6. + ((-0.005) * y))))), 60.}, {((-50.) + (10. * sin((5.36 + ((-0.005) * y))))), 53.6}, {((-50.) + (10. * sin((4.72 + ((-0.005) * y))))), 47.2}, {((-50.) + (10. * sin((4.08 + ((-0.005) * y))))), 40.8}, {((-50.) + (10. * sin((3.44 + ((-0.005) * y))))), 34.4}, {((-50.) + (10. * sin((2.8 + ((-0.005) * y))))), 28.}, {((-50.) + (10. * sin((2.16 + ((-0.005) * y))))), 21.6}, {((-50.) + (10. * sin((1.52 + ((-0.005) * y))))), 15.2}, {((-50.) + (10. * sin((0.88 + ((-0.005) * y))))), 8.8}, {((-50.) + (10. * sin((0.24 + ((-0.005) * y))))), 2.4}, {((-50.) + ((-10.) * sin((0.4 + (0.005 * y))))), (-4.)}, {((-50.) + ((-10.) * sin((1.04 + (0.005 * y))))), (-10.4)}, {((-50.) + ((-10.) * sin((1.68 + (0.005 * y))))), (-16.8)}, {((-50.) + ((-10.) * sin((2.32 + (0.005 * y))))), (-23.2)}, {((-50.) + ((-10.) * sin((2.96 + (0.005 * y))))), (-29.6)}, {((-50.) + ((-10.) * sin((3.6 + (0.005 * y))))), (-36.)}, {((-50.) + ((-10.) * sin((4.24 + (0.005 * y))))), (-42.4)}, {((-50.) + ((-10.) * sin((4.88 + (0.005 * y))))), (-48.8)}, {((-50.) + ((-10.) * sin((5.52 + (0.005 * y))))), (-55.2)}, {((-50.) + ((-10.) * sin((6.16 + (0.005 * y))))), (-61.6)}, {((-50.) + ((-10.) * sin((6.8 + (0.005 * y))))), (-68.)}, {((-50.) + ((-10.) * sin((7.44 + (0.005 * y))))), (-74.4)}, {((-50.) + ((-10.) * sin((8.08 + (0.005 * y))))), (-80.8)}, {((-50.) + ((-10.) * sin((8.72 + (0.005 * y))))), (-87.2)}, {((-50.) + ((-10.) * sin((9.36 + (0.005 * y))))), (-93.6)}, {((-50.) + ((-10.) * sin((10. + (0.005 * y))))), (-100.)}})
              , color = DynamicSelect({230, 46, 46}, if heatingSignal then {247, 135, 115} else if coolingSignal then {48, 175, 244} else {255, 255, 255}), thickness = 10, smooth = Smooth.Bezier),
            Polygon(visible = true, origin = {-125, 41.667}, lineColor = DynamicSelect({0, 0, 0}, if heatingSignal then {247, 135, 115} else if coolingSignal then {48, 175, 244} else {255, 255, 255}), fillColor = DynamicSelect({230, 46, 46}, if heatingSignal then {247, 135, 115} else if coolingSignal then {48, 175, 244} else {255, 255, 255}), pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-25, -16.667}, {0, 33.333}, {25, -16.667}}),
            Polygon(visible = true, origin = {-50, 41.667}, lineColor = DynamicSelect({0, 0, 0}, if heatingSignal then {247, 135, 115} else if coolingSignal then {48, 175, 244} else {255, 255, 255}), fillColor = DynamicSelect({230, 46, 46}, if heatingSignal then {247, 135, 115} else if coolingSignal then {48, 175, 244} else {255, 255, 255}), pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-25, -16.667}, {0, 33.333}, {25, -16.667}}),
            Polygon(visible = true, origin = {25, 41.667}, lineColor = DynamicSelect({0, 0, 0}, if heatingSignal then {247, 135, 115} else if coolingSignal then {48, 175, 244} else {255, 255, 255}), fillColor = DynamicSelect({230, 46, 46}, if heatingSignal then {247, 135, 115} else if coolingSignal then {48, 175, 244} else {255, 255, 255}), pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-25, -16.667}, {0, 33.333}, {25, -16.667}})
          }
        ) );
    end Heat;
    annotation(
      Icon(
        coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}),
        graphics = {
          Rectangle(visible = true, lineColor = {200, 200, 200}, fillColor = {248, 248, 248}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100, -100}, {100, 100}}, radius = 25),
          Rectangle(visible = true, lineColor = {128, 128, 128}, extent = {{-100, -100}, {100, 100}}, radius = 25)
        }
      )
    );
  end Components;

  package Utilities "Sublibrary of utility functions for visualization and data transformation"
    function temperatureToRGB 
      "Converts a temperature value to a color using RGB representation. Useful for heat maps or visual indicators."
      
      input Real Tmin "Minimum temperature (used for lower bound of color scale)";
      input Real Tmax "Maximum temperature (used for upper bound of color scale)";
      input Real T "Current temperature to map";
      output Real[3] RGB 
        "RGB color array scaled from 0 to 255. Maps Tmin to slate blue (106, 90, 205) and Tmax to orange (255, 165, 0)";
      
    protected 
      Real ratio "Normalized value of temperature between Tmin and Tmax";
      
    algorithm 
      // Clamp T between Tmin and Tmax
      if T < Tmin then
        ratio := 0;
      elseif T > Tmax then
        ratio := 1;
      else
        ratio := (T - Tmin) / (Tmax - Tmin);
      end if;

      // Interpolate between slate blue and orange
      RGB[1] := integer(106 + ratio * (255 - 106)); // Red
      RGB[2] := integer(90 + ratio * (165 - 90));   // Green
      RGB[3] := integer(205 + ratio * (0 - 205));   // Blue
        annotation(
        Documentation(
          info = "<html><p>This function converts a temperature value into an RGB color. It's useful when you want to visualize temperature data&mdash;like creating color maps, heatmaps, or intuitive visual indicators in a simulation.</p>
<p>The function linearly interpolates the temperature&nbsp;<code>T</code> between <code>Tmin</code> and <code>Tmax</code>.</p>
<ul>
<li>
<p>When <code>T = Tmin</code>, the color is <strong>slate blue</strong> (RGB = [106, 90, 205]).</p>
</li>
<li>
<p>When <code>T = Tmax</code>, the color is <strong>orange</strong> (RGB = [255, 165, 0]).</p>
</li>
<li>
<p>For temperatures in between, it smoothly blends the two colors.</p>
</li>
</ul>
<p><strong>Example usage</strong></p>
<p><br>Imagine you are simulating a heat exchanger and want to show the temperature of a fluid line in color:</p>
<pre><code class=\"language-modelica\">Real[3] color;
color := temperatureToRGB(273.15, 373.15, 300); // Convert 300K to RGB color
</code></pre>
<p>This might return something like <code>[150, 110, 160]</code>, depending on the scale.</p>
<p><strong>Good to know</strong></p>
<ul>
<li>
<p>Values below <code>Tmin</code> are clamped to the blue color.</p>
</li>
<li>
<p>Values above <code>Tmax</code> are clamped to the orange color.</p>
</li>
<li>
<p>You can use the resulting RGB values in visualization libraries or tools that support dynamic coloring.</p>
</li>
</ul></html>"
        ),
        Icon(
          coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}),
          graphics = {
            Ellipse(visible = true, lineColor = {0, 114, 195}, fillColor = {0, 114, 195}, fillPattern = FillPattern.Solid, extent = {{-100, -100}, {100, 100}}),
            Text(visible = true, origin = {0, -5}, textColor = {255, 255, 255}, extent = {{-90, -80}, {90, 80}}, textString = "f")
          }
        )
      );

    end temperatureToRGB;
    annotation(
      Icon(
        coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}),
        graphics = {
          Rectangle(visible = true, lineColor = {200, 200, 200}, fillColor = {248, 248, 248}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100, -100}, {100, 100}}, radius = 25),
          Rectangle(visible = true, lineColor = {128, 128, 128}, extent = {{-100, -100}, {100, 100}}, radius = 25)
        }
      )
    );
  end Utilities;
  annotation(
    uses(
      Modelica(
        version = "4.0.0"
      )
    ),
    Documentation(
      info = "<html><p>A batch process is a production method in which materials are handled in defined, finite quantities (batches), typically through multiple steps such as filling, heating, reacting, cooling, and draining. Industries such as pharmaceuticals, chemicals, and food processing, where precision, consistency, and safety are critical use this type of processing. The example combines models of physical processes&mdash;such as fluid flow and heat transfer&mdash;with statecharts to manage event-driven actions, effectively mimicking real-world industrial process automation.</p>
<div>
<p>In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at&nbsp;<a href=\"http://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a></p>
<p>For the full example, open the accompanying notebook&nbsp;<a href=\"modelica://BatchProcess/BatchProcess.nb\">BatchProcess.nb</a>.</p>
</div></html>"
    ),
    Icon(
      coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}),
      graphics = {
        Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90, -90}, {90, 90}}, radius = 25),
        Text(visible = true, origin = {12, 20.833}, textColor = {255, 255, 255}, extent = {{-102, -70.833}, {78, 29.167}}, textString = "Ex", fontName = "Arial")
      }
    )
  );
end BatchProcess;
